@@ -3,7 +3,6 @@ import uuid
 from typing import Any, Dict, List, Optional, TypeVar, Callable, Type
 from typing_extensions import Self
 from dotenv import load_dotenv
-import litellm
 
 from pydantic import UUID4, BaseModel, Field, InstanceOf, PrivateAttr, model_validator, field_validator
 from pydantic_core import PydanticCustomError
@@ -23,25 +22,6 @@ load_dotenv(override=True)
 T = TypeVar("T", bound="Agent")
 
 
-# def mock_agent_ops_provider():
-#     def track_agent(*args, **kwargs):
-#         def noop(f):
-#             return f
-#         return noop
-#     return track_agent
-
-# track_agent = mock_agent_ops_provider()
-
-# agentops = None
-# if os.environ.get("AGENTOPS_API_KEY"):
-#     try:
-#         from agentops import track_agent
-#     except ImportError:
-#         track_agent = mock_agent_ops_provider()
-# else:
-#    track_agent = mock_agent_ops_provider()
-
-
 class TokenProcess:
     total_tokens: int = 0
     prompt_tokens: int = 0
@@ -76,10 +56,7 @@ class TokenProcess:
 # @track_agent()
 class Agent(BaseModel):
     """
-    Agent class that run on LLM.
-    Agents execute tasks alone or in the team, using RAG tools and knowledge base if any.
-    Agents will prioritize team tasks when they belong to the team.
-    * (Temp) Comment out all the optional fields except for Team and LLM settings for convenience.
+    A Pydantic class to store an agent object. Agents must have `role` and `goal` to start.
     """
 
     __hash__ = object.__hash__
@@ -115,7 +92,7 @@ class Agent(BaseModel):
     user_prompt_template: Optional[str] = Field(default=None, description="user prompt template")
 
     # task execution rules
-    team: Optional[List[Any]] = Field(default=None, description="Team to which the agent belongs")
+    network: Optional[List[Any]] = Field(default=None, description="store a list of agent networks that the agent belong as a member")
     allow_delegation: bool = Field(default=False,description="if the agent can delegate the task to another agent or ask some help")
     max_retry_limit: int = Field(default=2 ,description="max. number of retry for the task execution when an error occurs")
     maxit: Optional[int] = Field(default=25,description="max. number of total optimization loops conducted when an error occurs")
@@ -132,8 +109,6 @@ class Agent(BaseModel):
 
     # cache, error, ops handling
     formatting_errors: int = Field(default=0, description="number of formatting errors.")
-    agent_ops_agent_name: str = None
-    agent_ops_agent_id: str = None
 
 
     @field_validator("id", mode="before")
@@ -161,9 +136,8 @@ class Agent(BaseModel):
     @model_validator(mode="after")
     def set_up_llm(self) -> Self:
         """
-        Set up `llm` and `function_calling_llm` as valid LLM objects using the given values.
+        Set up `llm` and `function_calling_llm` as valid LLM objects using the given kwargs.
         """
-        self.agent_ops_agent_name = self.role
         self.llm = self._convert_to_llm_object(llm=self.llm)
 
         function_calling_llm = self.function_calling_llm if self.function_calling_llm else self.llm if self.llm else None
@@ -177,12 +151,15 @@ class Agent(BaseModel):
         return self
 
 
-    def _convert_to_llm_object(self, llm: Any | None) -> LLM:
+    def _convert_to_llm_object(self, llm: Any = None) -> LLM:
         """
         Convert the given value to LLM object.
         When `llm` is dict or self.llm_config is not None, add these values to the LLM object after validating them.
         """
-        llm = llm if llm is not None else DEFAULT_MODEL_NAME
+        llm = llm if llm else self.llm if self.llm else DEFAULT_MODEL_NAME
+
+        if not llm:
+            pass
 
         match llm:
             case LLM():
@@ -274,7 +251,7 @@ class Agent(BaseModel):
             tool_list = []
 
             for item in self.tools:
-                if isinstance(item, Tool):
+                if isinstance(item, Tool) or isinstance(item, ToolSet):
                     tool_list.append(item)
 
                 elif isinstance(item, dict) and "func" in item:
@@ -380,16 +357,16 @@ class Agent(BaseModel):
         Set up memories: stm, ltm, and um
         """
 
-        if self.use_memory == True:
-            self.long_term_memory = self.long_term_memory if self.long_term_memory else LongTermMemory()
-            self.short_term_memory = self.short_term_memory if self.short_term_memory else ShortTermMemory(agent=self, embedder_config=self.embedder_config)
+        # if self.use_memory == True:
+        self.long_term_memory = self.long_term_memory if self.long_term_memory else LongTermMemory()
+        self.short_term_memory = self.short_term_memory if self.short_term_memory else ShortTermMemory(agent=self, embedder_config=self.embedder_config)
 
-            if hasattr(self, "memory_config") and self.memory_config is not None:
-                user_id = self.memory_config.get("user_id", None)
-                if user_id:
-                    self.user_memory = self.user_memory if self.user_memory else UserMemory(agent=self, user_id=user_id)
-            else:
-                self.user_memory = None
+        if hasattr(self, "memory_config") and self.memory_config is not None:
+            user_id = self.memory_config.get("user_id", None)
+            if user_id:
+                self.user_memory = self.user_memory if self.user_memory else UserMemory(agent=self, user_id=user_id)
+        else:
+            self.user_memory = None
 
         return self
 
@@ -402,6 +379,77 @@ class Agent(BaseModel):
             pass
 
 
+    def update_llm(self, llm: Any = None, llm_config: Optional[Dict[str, Any]] = None) -> Self:
+        """
+        Update llm and llm_config of the exsiting agent. (Other conditions will remain the same.)
+        """
+
+        if not llm and not llm_config:
+            self._logger.log(level="error", message="Missing llm or llm_config values to update", color="red")
+            pass
+
+        self.llm = llm
+        if llm_config:
+            if self.llm_config:
+                self.llm_config.update(llm_config)
+            else:
+                self.llm_config = llm_config
+
+        return self.set_up_llm()
+
+
+    def update(self, **kwargs) -> Self:
+        """
+        Update the existing agent. Address variables that require runnning set_up_x methods first, then update remaining variables.
+        """
+
+        if not kwargs:
+            self._logger.log(level="error", message="Missing values to update", color="red")
+            return self
+
+        for k, v in kwargs.items():
+            match k:
+                case "tools":
+                    self.tools = kwargs.get(k, self.tools)
+                    self.set_up_tools()
+
+                case "role" | "goal":
+                    self.role = kwargs.get("role", self.role)
+                    self.goal = kwargs.get("goal", self.goal)
+                    if not self.backstory:
+                        self.set_up_backstory()
+
+                    if self.backstory:
+                        self.backstory += f"new role: {self.role}, new goal: {self.goal}"
+
+                case "max_rpm":
+                    self.max_rpm = kwargs.get(k, self.max_rpm)
+                    self.set_up_rpm()
+
+                case "knowledge_sources":
+                    self.knowledge_sources = kwargs.get("knowledge_sources", self.knowledge_sources)
+                    self.set_up_knowledge()
+
+                case "use_memory" | "memory_config":
+                    self.use_memory = kwargs.get("use_memory", self.use_memory)
+                    self.memory_config = kwargs.get("memory_config", self.memory_config)
+                    self.set_up_memory()
+
+                case "llm" | "llm_config":
+                    self.llm = kwargs.get("llm", self.llm)
+                    self.llm_config = kwargs.get("llm_config", self.llm_config)
+                    self.update_llm(llm=self.llm, llm_config=self.llm_config)
+
+                case _:
+                    try:
+                        setattr(self, k, v)
+                    except Exception as e:
+                        self._logger.log(level="error", message=f"Failed to update the key: {k} We'll skip. Error: {str(e)}", color="red")
+                        pass
+
+        return self
+
+
     def invoke(
         self,
         prompts: str,
@@ -463,14 +511,11 @@ class Agent(BaseModel):
                 raise ValueError("Invalid response from LLM call - None or empty.")
 
 
-
-
-    def execute_task(self, task, context: Optional[str] = None, task_tools: Optional[List[Tool | ToolSet]] = list()) -> str:
+    def execute_task(self, task, context: Optional[Any] = None, task_tools: Optional[List[Tool | ToolSet]] = list()) -> str:
         """
-        Execute the task and return the response in string.
-        The agent utilizes the tools in task or their own tools if the task.can_use_agent_tools is True.
-        The agent must consider the context to excute the task as well when it is given.
+        Format a task prompt, adding context from knowledge and memory (if given), and invoke LLM.
         """
+
         from versionhq.task.model import Task
         from versionhq.knowledge._utils import extract_knowledge_context
 
@@ -480,9 +525,7 @@ class Agent(BaseModel):
         if self.max_rpm and self._rpm_controller:
             self._rpm_controller._reset_request_count()
 
-        task_prompt = task.prompt(model_provider=self.llm.provider)
-        if context is not task.prompt_context:
-            task_prompt += context
+        task_prompt = task._prompt(model_provider=self.llm.provider, context=context)
 
         if self._knowledge:
             agent_knowledge = self._knowledge.query(query=[task_prompt,], limit=5)
@@ -495,13 +538,15 @@ class Agent(BaseModel):
             contextual_memory = ContextualMemory(
                 memory_config=self.memory_config, stm=self.short_term_memory, ltm=self.long_term_memory, um=self.user_memory
             )
-            memory = contextual_memory.build_context_for_task(task=task, context=context)
+            context_str = task._draft_context_prompt(context=context)
+            query = f"{task.description} {context_str}".strip()
+            memory = contextual_memory.build_context_for_task(query=query)
             if memory.strip() != "":
                 task_prompt += memory.strip()
 
 
         ## comment out for now
-        # if self.team and self.team._train:
+        # if self.network and self.network._train:
         #     task_prompt = self._training_handler(task_prompt=task_prompt)
         # else:
         #     task_prompt = self._use_trained_data(task_prompt=task_prompt)
@@ -531,5 +576,27 @@ class Agent(BaseModel):
         return raw_response
 
 
+    def start(self, context: Any = None) -> Any | None:
+        """
+        Defines and executes a task when it is not given and returns TaskOutput object.
+        """
+
+        if not self.goal or not self.role:
+            return None
+
+        from versionhq.task.model import Task, ResponseField
+
+        class Output(BaseModel):
+            result: str
+            steps: list[str]
+
+        task = Task(
+            description=f"Generate a simple result in a sentence to achieve the goal: {self.goal}. If needed, list up necessary steps in concise manner.",
+            pydantic_output=Output
+        )
+        res = task.execute(agent=self, context=context)
+        return res
+
+
     def __repr__(self):
-        return f"Agent(role={self.role}, goal={self.goal}, backstory={self.backstory})"
+        return f"Agent(role={self.role}, goal={self.goal}"
