@@ -3,6 +3,8 @@ import threading
 import datetime
 import uuid
 import inspect
+import enum
+from textwrap import dedent
 from concurrent.futures import Future
 from hashlib import md5
 from typing import Any, Dict, List, Set, Optional, Callable, Type
@@ -11,18 +13,25 @@ from typing_extensions import Annotated, Self
 from pydantic import UUID4, BaseModel, Field, PrivateAttr, field_validator, model_validator, InstanceOf, field_validator
 from pydantic_core import PydanticCustomError
 
-
+import versionhq as vhq
 from versionhq.task.log_handler import TaskOutputStorageHandler
 from versionhq.task.evaluate import Evaluation, EvaluationItem
 from versionhq.tool.model import Tool, ToolSet
 from versionhq._utils import process_config, Logger
 
 
+class TaskExecutionType(enum.Enum):
+    """
+    Enumeration to store task execution types of independent tasks without dependencies.
+    """
+    SYNC = 1
+    ASYNC = 2
+
+
 class ResponseField(BaseModel):
     """
-    A class to store the response format and schema that will cascade to the LLM.
-    The `config` field can store additional params:
-    https://community.openai.com/t/official-documentation-for-supported-schemas-for-response-format-parameter-in-calls-to-client-beta-chats-completions-parse/932422/3
+    A class to store a response format that will generate a JSON schema.
+    One layer of nested child is acceptable.
     """
 
     title: str = Field(default=None, description="title of the field")
@@ -31,7 +40,7 @@ class ResponseField(BaseModel):
     properties: Optional[List[BaseModel]] = Field(default=None, description="store dict items in ResponseField format")
     required: bool = Field(default=True)
     nullable: bool = Field(default=False)
-    config: Optional[Dict[str, Any]] = Field(default=None, description="additional rules")
+    config: Optional[Dict[str, Any]] = Field(default_factory=dict, description="additional rules")
 
 
     @model_validator(mode="after")
@@ -57,36 +66,44 @@ class ResponseField(BaseModel):
 
     def _format_props(self) -> Dict[str, Any]:
         """
-        Structure valid properties. We accept 2 nested objects.
+        Structure valid properties from the ResponseField object. 1 layer of nested child is accepted.
         """
         from versionhq.llm.llm_vars import SchemaType
 
         schema_type = SchemaType(type=self.data_type).convert()
         props: Dict[str, Any] = {}
 
-        if self.data_type is list and self.items is not dict:
-            props = {
-                "type": schema_type,
-                "items": { "type": SchemaType(type=self.items).convert() },
-            }
+        if self.data_type is list:
+            if self.items is dict:
+                nested_p, nested_r = dict(), list()
 
-        elif self.data_type is list and self.items is dict:
-            nested_p, nested_r = dict(), list()
+                if self.properties:
+                    for item in self.properties:
+                        nested_p.update(**item._format_props())
+                        nested_r.append(item.title)
 
-            if self.properties:
-                for item in self.properties:
-                    nested_p.update(**item._format_props())
-                    nested_r.append(item.title)
-
-            props = {
-                "type": schema_type,
-                "items": {
-                    "type": SchemaType(type=self.items).convert(),
-                    "properties": nested_p,
-                    "required": nested_r,
-                    "additionalProperties": False
+                props = {
+                    "type": schema_type,
+                    "items": {
+                        "type": SchemaType(type=self.items).convert(),
+                        "properties": nested_p,
+                        "required": nested_r,
+                        "additionalProperties": False
+                    }
                 }
-            }
+
+            elif self.items is list:
+                props = {
+                    "type": schema_type,
+                    "items": { "type": SchemaType(type=self.items).convert(), "items": { "type": SchemaType(type=str).convert() }},
+                }
+
+            else:
+                props = {
+                    "type": schema_type,
+                    "items": { "type": SchemaType(type=self.items).convert() },
+                }
+
 
         elif self.data_type is dict:
             p, r = dict(), list()
@@ -133,9 +150,16 @@ class ResponseField(BaseModel):
             return value
 
 
+    def _annotate(self, value: Any) -> Annotated:
+        """
+        Address Pydantic's `create_model`
+        """
+        return Annotated[self.type, value] if isinstance(value, self.type) else Annotated[str, str(value)]
+
+
     def create_pydantic_model(self, result: Dict, base_model: InstanceOf[BaseModel] | Any) -> Any:
         """
-        Create a Pydantic model from the given result
+        Create a Pydantic model from the given result.
         """
         for k, v in result.items():
             if k is not self.title:
@@ -148,14 +172,6 @@ class ResponseField(BaseModel):
         return base_model
 
 
-    def _annotate(self, value: Any) -> Annotated:
-        """
-        Address Pydantic's `create_model`
-        """
-        return Annotated[self.type, value] if isinstance(value, self.type) else Annotated[str, str(value)]
-
-
-
 class TaskOutput(BaseModel):
     """
     A class to store the final output of the given task in raw (string), json_dict, and pydantic class formats.
@@ -170,31 +186,32 @@ class TaskOutput(BaseModel):
     evaluation: Optional[InstanceOf[Evaluation]] = Field(default=None, description="store overall evaluation of the task output. passed to ltm")
 
 
-    def to_dict(self) -> Dict[str, Any] | None:
+    def to_context_prompt(self) -> str:
         """
-        Convert pydantic / raw output into dict and return the dict.
+        Returns response in string as a prompt context.
         """
-        return self.json_dict if self.json_dict is not None else self.pydantic.model_dump() if self.pydantic else None
+        context = ""
+        try:
+            context = json.dumps(self.json_dict)
+        except:
+            try:
+                if self.pydantic:
+                    context = self.pydantic.model_dump()
+            except:
+                context = self.raw
+        return context
 
 
-    def context_prompting(self) -> str:
-        """
-        When the task is called as context, return its output in concise string to add it to the prompt
-        """
-        return json.dumps(self.json_dict) if self.json_dict else self.raw[0: 127]
-
-
-    def evaluate(self, task, latency: int | float = None, tokens: int = None) -> Evaluation:
+    def evaluate(self, task) -> Evaluation:
         """
         Evaluate the output based on the criteria, score each from 0 to 1 scale, and raise suggestions for future improvement.
         """
         from versionhq.task.TEMPLATES.Description import EVALUATE
 
-        if not self.evaluation:
-            self.evaluation = Evaluation()
+        self.evaluation = Evaluation() if not self.evaluation else self.evaluation
 
-        self.evaluation.latency = latency if latency is not None else task.latency
-        self.evaluation.tokens = tokens if tokens is not None else task.tokens
+        # self.evaluation.latency = latency if latency is not None else task.latency
+        # self.evaluation.tokens = tokens if tokens is not None else task.tokens
 
         eval_criteria = task.eval_criteria if task.eval_criteria else  ["Overall competitiveness", ]
 
@@ -203,7 +220,7 @@ class TaskOutput(BaseModel):
                 description=EVALUATE.format(task_description=task.description, task_output=self.raw, eval_criteria=str(item)),
                 pydantic_output=EvaluationItem
             )
-            res = task_eval.execute_sync(agent=self.evaluation.responsible_agent)
+            res = task_eval.execute(agent=self.evaluation.eval_by)
 
             if res.pydantic:
                 item = EvaluationItem(score=res.pydantic.score, suggestion=res.pydantic.suggestion, criteria=res.pydantic.criteria)
@@ -231,7 +248,7 @@ class TaskOutput(BaseModel):
 
 
     @property
-    def json(self) -> Optional[str]:
+    def json_string(self) -> Optional[str]:
         return json.dumps(self.json_dict)
 
 
@@ -241,14 +258,10 @@ class TaskOutput(BaseModel):
 
 class Task(BaseModel):
     """
-    Task to be executed by agents or teams.
-    Each task must have a description.
-    Default response is JSON string that strictly follows `response_fields` - and will be stored in TaskOuput.raw / json_dict.
-    When `pydantic_output` is provided, we prioritize them and store raw (json string), json_dict, pydantic in the TaskOutput class.
+    A class that stores independent task information and handles task executions.
     """
 
     __hash__ = object.__hash__
-    _logger: Logger = PrivateAttr(default_factory=lambda: Logger(verbose=True))
     _original_description: str = PrivateAttr(default=None)
     _task_output_handler = TaskOutputStorageHandler()
     config: Optional[Dict[str, Any]] = Field(default=None, description="values to set on Task class")
@@ -257,23 +270,18 @@ class Task(BaseModel):
     name: Optional[str] = Field(default=None)
     description: str = Field(description="Description of the actual task")
 
-    # output
-    pydantic_output: Optional[Any] = Field(default=None, description="store a custom Pydantic class as response format")
-    response_fields: List[ResponseField] = Field(default_factory=list, description="store the list of ResponseFields to create the response format")
-    output: Optional[TaskOutput] = Field(default=None, description="store the final task output in TaskOutput class")
-
-    # task setup
-    context: Optional[List["Task"]] = Field(default=None, description="other tasks whose outputs should be used as context")
-    prompt_context: Optional[str] = Field(default=None)
+    # response format
+    pydantic_output: Optional[Type[BaseModel]] = Field(default=None, description="store Pydantic class as structured response format")
+    response_fields: Optional[List[ResponseField]] = Field(default_factory=list, description="store list of ResponseField as structured response format")
 
     # tool usage
     tools: Optional[List[ToolSet | Tool | Any]] = Field(default_factory=list, description="tools that the agent can use aside from their tools")
-    can_use_agent_tools: bool = Field(default=False, description="whether the agent can use their own tools when executing the task")
+    can_use_agent_tools: bool = Field(default=True, description="whether the agent can use their own tools when executing the task")
     tool_res_as_final: bool = Field(default=False, description="when set True, tools res will be stored in the `TaskOutput`")
 
-    # execution rules
+    # executing
+    execution_type: TaskExecutionType = Field(default=TaskExecutionType.SYNC)
     allow_delegation: bool = Field(default=False, description="ask other agents for help and run the task instead")
-    async_execution: bool = Field(default=False,description="whether the task should be executed asynchronously or not")
     callback: Optional[Callable] = Field(default=None, description="callback to be executed after the task is completed.")
     callback_kwargs: Optional[Dict[str, Any]] = Field(default_factory=dict, description="kwargs for the callback when the callback is callable")
 
@@ -281,12 +289,13 @@ class Task(BaseModel):
     should_evaluate: bool = Field(default=False, description="True to run the evaluation flow")
     eval_criteria: Optional[List[str]] = Field(default_factory=list, description="criteria to evaluate the outcome. i.e., fit to the brand tone")
 
-    # recording
-    processed_by_agents: Set[str] = Field(default_factory=set, description="store responsible agents' roles")
-    tools_errors: int = 0
+    # recording !# REFINEME - eval_callbacks
+    processed_agents: Set[str] = Field(default_factory=set, description="store roles of the agents that executed the task")
+    tool_errors: int = 0
     delegations: int = 0
     latency: int | float = 0 # job latency in sec
     tokens: int = 0 # tokens consumed
+    output: Optional[TaskOutput] = Field(default=None, description="store the final task output in TaskOutput class")
 
 
     @model_validator(mode="before")
@@ -328,13 +337,6 @@ class Task(BaseModel):
         return self
 
 
-    @model_validator(mode="after")
-    def backup_description(self):
-        if self._original_description == None:
-            self._original_description = self.description
-        return self
-
-
     def _draft_output_prompt(self, model_provider: str) -> str:
         """
         Draft prompts on the output format by converting `
@@ -360,35 +362,61 @@ Your response MUST be a valid JSON string that strictly follows the response for
 Response format: {response_format}
 Ref. Output image: {output_formats_to_follow}
 """
-
         else:
-            output_prompt = "Return your response as a valid JSON serializable string, enclosed in double quotes. Do not use single quotes, trailing commas, or other non-standard JSON syntax."
+            output_prompt = "You MUST Return your response as a valid JSON serializable string, enclosed in double quotes. Do not use single quotes, trailing commas, or other non-standard JSON syntax."
 
-        return output_prompt
+        return dedent(output_prompt)
 
 
-    def prompt(self, model_provider: str = None, customer: str = None, product_overview: str = None) -> str:
+    def _draft_context_prompt(self, context: Any) -> str:
+        """
+        Create a context prompt from the given context in any format: a task object, task output object, list, dict.
+        """
+
+        context_to_add = None
+        if not context:
+            # Logger().log(level="error", color="red", message="Missing a context to add to the prompt. We'll return ''.")
+            return context_to_add
+
+        match context:
+            case str():
+                context_to_add = context
+
+            case Task():
+                if not context.output:
+                    res = context.execute()
+                    context_to_add = res.to_context_prompt()
+
+                else:
+                    context_to_add = context.output.raw
+
+            case TaskOutput():
+                context_to_add = context.to_context_prompt()
+
+
+            case dict():
+                context_to_add = str(context)
+
+            case list():
+                res = ", ".join([self._draft_context_prompt(context=item) for item in context])
+                context_to_add = res
+
+            case _:
+                pass
+
+        return dedent(context_to_add)
+
+
+    def _prompt(self, model_provider: str = None, context: Optional[Any] = None) -> str:
         """
         Format the task prompt and cascade it to the agent.
-        When the task has context, add context prompting of all the tasks in the context.
-        When we have cusotmer/product info, add them to the prompt.
         """
-
         output_prompt = self._draft_output_prompt(model_provider=model_provider)
-        task_slices = [self.description, output_prompt,]
+        task_slices = [self.description, output_prompt, ]
 
-        if self.context:
-            context_outputs = "\n".join([task.output.context_prompting() if hasattr(task, "output") else "" for task in self.context])
-            task_slices.insert(len(task_slices), f"Consider the following context when responding: {context_outputs}")
-
-        if self.prompt_context:
-            task_slices.insert(len(task_slices), f"Consider the following context when responding: {self.prompt_context}")
-
-        if customer:
-            task_slices.insert(len(task_slices), f"Customer to address: {customer}")
-
-        if product_overview:
-            task_slices.insert(len(task_slices), f"Product to promote: {product_overview}")
+        if context:
+            context_prompt = self._draft_context_prompt(context=context)
+            task_slices.insert(len(task_slices), f"Consider the following context when responding: {context_prompt}")
 
         return "\n".join(task_slices)
 
@@ -411,11 +439,7 @@ Ref. Output image: {output_formats_to_follow}
                 properties, required_fields = {}, []
                 for i, item in enumerate(self.response_fields):
                     if item:
-                        if item.data_type is dict:
-                            properties.update(item._format_props())
-                        else:
-                            properties.update(item._format_props())
-
+                        properties.update(item._format_props())
                         required_fields.append(item.title)
 
                 response_schema = {
@@ -424,7 +448,6 @@ Ref. Output image: {output_formats_to_follow}
                     "required": required_fields,
                     "additionalProperties": False,
                 }
-
                 response_format = {
                     "type": "json_schema",
                     "json_schema": { "name": "outcome", "schema": response_schema }
@@ -432,7 +455,7 @@ Ref. Output image: {output_formats_to_follow}
 
 
             elif self.pydantic_output:
-                response_format = StructuredOutput(response_format=self.pydantic_output)._format()
+                response_format = StructuredOutput(response_format=self.pydantic_output, provider=model_provider)._format()
 
             return response_format
 
@@ -443,8 +466,8 @@ Ref. Output image: {output_formats_to_follow}
         """
 
         if raw is None or raw == "":
-            self._logger.log(level="warning", message="The model returned an empty response. Returning an empty dict.", color="yellow")
-            output = { "output": "n.a." }
+            Logger().log(level="warning", message="The model returned an empty response. Returning an empty dict.", color="yellow")
+            output = { "output": "" }
             return output
 
         try:
@@ -455,7 +478,7 @@ Ref. Output image: {output_formats_to_follow}
                 return output
 
             else:
-                r = str(raw).replace("{'", '{"').replace("{ '", '{"').replace("': '", '": "').replace("'}", '"}').replace("' }", '"}').replace("', '", '", "').replace("['", '["').replace("[ '", '[ "').replace("']", '"]').replace("' ]", '" ]').replace("{\n'", '{"').replace("{\'", '{"').replace("true", "True").replace("false", "False")
+                r = str(raw).strip().replace("{'", '{"').replace("{ '", '{"').replace("': '", '": "').replace("'}", '"}').replace("' }", '"}').replace("', '", '", "').replace("['", '["').replace("[ '", '[ "').replace("']", '"]').replace("' ]", '" ]').replace("{\n'", '{"').replace("{\'", '{"').replace("true", "True").replace("false", "False").replace('\"', "'")
                 j = json.dumps(eval(r))
                 output = json.loads(j)
 
@@ -484,7 +507,6 @@ Ref. Output image: {output_formats_to_follow}
 
             for k, v in json_dict.items():
                 setattr(output_pydantic, k, v)
-
         except:
             pass
 
@@ -493,8 +515,10 @@ Ref. Output image: {output_formats_to_follow}
 
     def interpolate_inputs(self, inputs: Dict[str, Any]) -> None:
         """
-        Interpolate inputs into the task description and expected output.
+        Interpolate inputs into the task description.
         """
+        self._original_description = self.description
+
         if inputs:
             self.description = self._original_description.format(**inputs)
 
@@ -532,35 +556,51 @@ Ref. Output image: {output_formats_to_follow}
                 )
 
         except AttributeError as e:
-            self._logger.log(level="error", message=f"Missing attributes for long term memory: {str(e)}", color="red")
+            Logger().log(level="error", message=f"Missing attributes for long term memory: {str(e)}", color="red")
             pass
 
         except Exception as e:
-            self._logger.log(level="error", message=f"Failed to add to the memory: {str(e)}", color="red")
+            Logger().log(level="error", message=f"Failed to add to the memory: {str(e)}", color="red")
             pass
 
 
+    def _build_agent_from_task(self, task_description: str = None) -> InstanceOf["vhq.Agent"]:
+        task_description = task_description if task_description else self.description
+        if not task_description:
+            Logger().log(level="error", message="Task is missing the description.", color="red")
+            pass
+
+        agent = vhq.Agent(goal=task_description, role=task_description, maxit=1) #! REFINEME
+        return agent
+
+
     # task execution
-    def execute_sync(self, agent, context: Optional[str | List[Any]] = None) -> TaskOutput:
+    def execute(
+            self, type: TaskExecutionType = None, agent: Optional["vhq.Agent"] = None, context: Optional[Any] = None
+            ) -> TaskOutput | Future[TaskOutput]:
         """
-        Execute the task synchronously.
-        When the task has context, make sure we have executed all the tasks in the context first.
+        A main method to handle task execution. Build an agent when the agent is not given.
         """
+        type = type if type else  self.execution_type if self.execution_type else TaskExecutionType.SYNC
 
-        if self.context:
-            if isinstance(self.context, list):
-                for task in self.context:
-                    if isinstance(task, Task) and task.output is None:
-                        task._execute_core(agent, context)
+        if not agent:
+            agent = self._build_agent_from_task(task_description=self.description)
 
+        match type:
+            case TaskExecutionType.SYNC:
+                return self._execute_sync(agent=agent, context=context)
+
+            case TaskExecutionType.ASYNC:
+                return self._execute_async(agent=agent, context=context)
+
+
+    def _execute_sync(self, agent, context: Optional[Any] = None) -> TaskOutput:
+        """Executes the task synchronously."""
         return self._execute_core(agent, context)
 
 
-    def execute_async(self, agent, context: Optional[str] = None) -> Future[TaskOutput]:
-        """
-        Execute the task asynchronously.
-        """
-
+    def _execute_async(self, agent, context: Optional[Any] = None) -> Future[TaskOutput]:
+        """Executes the task asynchronously."""
         future: Future[TaskOutput] = Future()
         threading.Thread(daemon=True, target=self._execute_task_async, args=(agent, context, future)).start()
         return future
@@ -568,24 +608,23 @@ Ref. Output image: {output_formats_to_follow}
 
     def _execute_task_async(self, agent, context: Optional[str], future: Future[TaskOutput]) -> None:
         """
-        Execute the task asynchronously with context handling.
+        Executes the task asynchronously with context handling.
         """
-
         result = self._execute_core(agent, context)
         future.set_result(result)
 
 
-    def _execute_core(self, agent, context: Optional[str]) -> TaskOutput:
+    def _execute_core(self, agent, context: Optional[Any]) -> TaskOutput:
         """
-        Execute the given task with the given agent.
-        Handle 1. agent delegation, 2. tools, 3. context to consider, and 4. callbacks
+        A core method for task execution.
+        Handles 1. agent delegation, 2. tools, 3. context to add to the prompt, and 4. callbacks.
         """
 
         from versionhq.agent.model import Agent
-        from versionhq.team.model import Team
+        from versionhq.agent_network.model import AgentNetwork
 
-        self.prompt_context = context
         task_output: InstanceOf[TaskOutput] = None
+        raw_output: str = None
         tool_output: str | list = None
         task_tools: List[List[InstanceOf[Tool]| InstanceOf[ToolSet] | Type[Tool]]] = []
         started_at, ended_at = datetime.datetime.now(), datetime.datetime.now()
@@ -595,15 +634,15 @@ Ref. Output image: {output_formats_to_follow}
                 if isinstance(item, ToolSet) or isinstance(item, Tool) or type(item) == Tool:
                     task_tools.append(item)
 
-        if self.allow_delegation:
+        if self.allow_delegation == True:
             agent_to_delegate = None
 
-            if hasattr(agent, "team") and isinstance(agent.team, Team):
-                if agent.team.managers:
-                    idling_manager_agents = [manager.agent for manager in agent.team.managers if manager.is_idling]
-                    agent_to_delegate = idling_manager_agents[0] if idling_manager_agents else agent.team.managers[0]
+            if hasattr(agent, "network") and isinstance(agent.network, AgentNetwork):
+                if agent.network.managers:
+                    idling_manager_agents = [manager.agent for manager in agent.network.managers if manager.is_idling]
+                    agent_to_delegate = idling_manager_agents[0] if idling_manager_agents else agent.network.managers[0]
                 else:
-                    peers = [member.agent for member in agent.team.members if member.is_manager == False and member.agent.id is not agent.id]
+                    peers = [member.agent for member in agent.network.members if member.is_manager == False and member.agent.id is not agent.id]
                     if len(peers) > 0:
                         agent_to_delegate = peers[0]
             else:
@@ -637,13 +676,13 @@ Ref. Output image: {output_formats_to_follow}
                 json_dict=json_dict_output
             )
 
-
         self.latency = (ended_at - started_at).total_seconds()
+        task_output.evaluation = Evaluation(latency=self.latency, tokens=self.tokens)
         self.output = task_output
-        self.processed_by_agents.add(agent.role)
+        self.processed_agents.add(agent.role)
 
-        if self.should_evaluate:
-            task_output.evaluate(task=self, latency=self.latency, tokens=self.tokens)
+        if self.should_evaluate and raw_output: # eval only when raw output exsits
+            task_output.evaluate(task=self)
 
         self._create_short_and_long_term_memories(agent=agent, task_output=task_output)
 
@@ -651,7 +690,7 @@ Ref. Output image: {output_formats_to_follow}
             kwargs = { **self.callback_kwargs, **task_output.json_dict }
             sig = inspect.signature(self.callback)
             valid_keys = [param.name for param in sig.parameters.values() if param.kind == param.POSITIONAL_OR_KEYWORD]
-            valid_kwargs = { k: kwargs[k] for k in valid_keys }
+            valid_kwargs = { k: kwargs[k] if  k in kwargs else None for k in valid_keys }
             callback_res = self.callback(**valid_kwargs)
             task_output.callback_output = callback_res
 
@@ -687,53 +726,3 @@ Task ID: {str(self.id)}
 "Description": {self.description}
 "Tools": {", ".join([tool.name for tool in self.tools])}
         """
-
-
-
-class ConditionalTask(Task):
-    """
-    A task that can be conditionally executed based on the output of another task.
-    When the `condition` return True, execute the task, else skipped with `skipped task output`.
-    """
-
-    condition: Callable[[TaskOutput], bool] = Field(
-        default=None,
-        description="max. number of retries for an agent to execute a task when an error occurs",
-    )
-
-
-    def __init__(self, condition: Callable[[Any], bool], **kwargs):
-        super().__init__(**kwargs)
-        self.condition = condition
-        self._logger = Logger(verbose=True)
-
-
-    def should_execute(self, context: TaskOutput) -> bool:
-        """
-        Decide whether the conditional task should be executed based on the provided context.
-        Return `True` if it should be executed.
-        """
-        return self.condition(context)
-
-
-    def get_skipped_task_output(self):
-        return TaskOutput(task_id=self.id, raw="", pydantic=None, json_dict={})
-
-
-    def _handle_conditional_task(self, task_outputs: List[TaskOutput], task_index: int, was_replayed: bool) -> Optional[TaskOutput]:
-        """
-        When the conditional task should be skipped, return `skipped_task_output` as task_output else return None
-        """
-
-        previous_output = task_outputs[task_index - 1] if task_outputs and len(task_outputs) > 1  else None
-
-        if previous_output and not self.should_execute(previous_output):
-            self._logger.log(level="warning", message=f"Skipping conditional task: {self.description}", color="yellow")
-            skipped_task_output = self.get_skipped_task_output()
-            self.output = skipped_task_output
-
-            if not was_replayed:
-                self._store_execution_log(self, task_index=task_index, was_replayed=was_replayed, inputs={})
-            return skipped_task_output
-
-        return None
